@@ -15,11 +15,12 @@
 #include "logging_p.h"
 #include <config-kglobalaccel.h>
 
+#include <KApplicationTrader>
 #include <KDesktopFile>
 #include <KFileUtils>
 #include <KPluginMetaData>
+#include <KSycoca>
 
-#include <KApplicationTrader>
 #include <QDBusConnection>
 #include <QDir>
 #include <QGuiApplication>
@@ -151,7 +152,7 @@ void GlobalShortcutsRegistry::migrateKHotkeys()
         file.desktopGroup().writeEntry("X-KDE-GlobalAccel-CommandShortcut", true);
         file.desktopGroup().writeEntry("StartupNotify", false);
 
-        _config.group(QStringLiteral("services")).group(uuid + QLatin1String(".desktop")).writeEntry("_launch", kglobalaccelEntry);
+        _config.group(QStringLiteral("services")).group(uuid + QLatin1String(".desktop")).writeEntry("_launch", key);
         _config.group(QStringLiteral("khotkeys")).revertToDefault(uuid);
     }
 }
@@ -258,6 +259,8 @@ GlobalShortcutsRegistry::GlobalShortcutsRegistry()
     if (_manager) {
         _manager->setEnabled(true);
     }
+
+    connect(KSycoca::self(), &KSycoca::databaseChanged, this, &GlobalShortcutsRegistry::refreshServices);
 }
 
 GlobalShortcutsRegistry::~GlobalShortcutsRegistry()
@@ -670,13 +673,26 @@ void GlobalShortcutsRegistry::loadSettings()
         actionComp->loadFromService();
     }
 
+    detectAppsWithShortcuts();
+}
+
+void GlobalShortcutsRegistry::detectAppsWithShortcuts()
+{
     auto appsWithShortcuts = KApplicationTrader::query([](const KService::Ptr &service) {
-        return !service->property<QString>(QStringLiteral("X-KDE-Shortcuts")).isEmpty();
+        if (!service->property<QString>(QStringLiteral("X-KDE-Shortcuts")).isEmpty()) {
+            return true;
+        }
+
+        const auto actions = service->actions();
+        return std::any_of(actions.cbegin(), actions.cend(), [](const KServiceAction &action) {
+            return !action.property<QStringList>(QStringLiteral("X-KDE-Shortcuts")).isEmpty();
+        });
     });
 
     for (auto service : appsWithShortcuts) {
         auto it = findByName(service->storageId());
         if (it != m_components.cend()) {
+            // already there
             continue;
         }
 
@@ -813,4 +829,33 @@ void GlobalShortcutsRegistry::writeSettings()
     _config.sync();
 }
 
+void GlobalShortcutsRegistry::refreshServices()
+{
+    // Remove shortcuts for no longer existing apps
+    auto it = std::remove_if(m_components.begin(), m_components.end(), [](const ComponentPtr &component) {
+        bool isService = component->uniqueName().endsWith(QLatin1String(".desktop"));
+
+        if (!isService) {
+            return false;
+        }
+
+        if (KService::serviceByStorageId(component->uniqueName())) {
+            // still there
+            return false;
+        }
+
+        if (!QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String("kglobalaccel/") + component->uniqueName()).isEmpty()) {
+            // still there
+            return false;
+        }
+
+        return true;
+    });
+
+    m_components.erase(it, m_components.end());
+
+    // Look for new apps with shortcuts
+    detectAppsWithShortcuts();
+}
+
 #include "moc_globalshortcutsregistry.cpp"
